--[[
This controls the configuration settings (and display of a sample set of
actions based on those settings) for each ActionSetLayout.  Each
BarActionSetLayout is associated with an BarActionSetLayoutConfig, and each
BarActionSetLayoutConfig refers to that same BarActionSetLayout instance.
--]]
uabSquareLayoutManager.LayoutConfig = uab.mixins.aceOO.Class(IActionSetLayoutConfig, uab.mixins.aceEvent)
uabSquareLayoutManager.LayoutConfig.count = 0

function uabSquareLayoutManager.LayoutConfig.prototype:init(layout, settings)
    uabSquareLayoutManager.LayoutConfig.super.prototype.init(self)

	self.settings = settings

	self:SettingsChanged()

	uabSquareLayoutManager.LayoutConfig.count = uabSquareLayoutManager.LayoutConfig.count + 1
	self.number = uabSquareLayoutManager.LayoutConfig.count
    self.layout = layout
end

function uabSquareLayoutManager.LayoutConfig.prototype:SettingsChanged(settings)
	if not self.settings then
		self.settings = {}
	end

    if not settings then
        settings = self.settings
    end
    
    self.settings.growClockwise = (settings and settings.growClockwise) or "true"
    self.settings.startLocation = (settings and settings.startLocation) or uab.constants.placement.top

    --[[
    Update the controls in the frame to show the new settings
    --]]
    self:SetGrowClockwise(self.settings.growClockwise)
    self:SetStartLocation(self.settings.startLocation)
end

function uabSquareLayoutManager.LayoutConfig.prototype:UseDefaults(bypassNotification)
	if not self.settings then
	    self.settings = {}
	end

    self.settings.growClockwise = "true"
    self.settings.startLocation = uab.constants.placement.top
	
    self:SettingsChanged()
	
    if not bypassNotification then
        self.layout:LayoutSettingsChanged()
    end
end

function uabSquareLayoutManager.LayoutConfig.prototype:MenuOptionArgs()
	--[[
	Build an AceOptions table of the options to be used to configure this
	config instance.  This table is inserted into a parent menu as a set of
	options.  If there are several options, just define them at the top level
	of the returned table.  Do not create a 'group' at the top level to enclose
	the options.
	--]]
	
	local options = {
        clockwise = {
            type = 'toggle',
            name = "Add buttons clockwise",
            desc = "As buttons are added, then should be added around the square in clockwise order",
            get = function()
                return self.settings.growClockwise == "true"
            end,
            set = function(v)
                if v then
					self:SetGrowClockwise("true")
				else
				    self:SetGrowClockwise("false")
                end
            end,
        },
        location = {
            type = 'text',
            name = "First button placement",
            desc = "Placement of the first button, relative to the cancel button",
            get = function()
                return uab.constants.PlacementDisplayValue(self.settings.startLocation)
            end,
            set = function(placement)
                if placement == uab.constants.PlacementDisplayValue(uab.constants.placement.topleft) then
                    self:SetStartLocation(uab.constants.placement.topleft)
				elseif placement == uab.constants.PlacementDisplayValue(uab.constants.placement.top) then
                	self:SetStartLocation(uab.constants.placement.top)
				elseif placement == uab.constants.PlacementDisplayValue(uab.constants.placement.topright) then
                	self:SetStartLocation(uab.constants.placement.topright)
				elseif placement == uab.constants.PlacementDisplayValue(uab.constants.placement.bottomleft) then
                	self:SetStartLocation(uab.constants.placement.bottomleft)
				elseif placement == uab.constants.PlacementDisplayValue(uab.constants.placement.bottom) then
                	self:SetStartLocation(uab.constants.placement.bottom)
				elseif placement == uab.constants.PlacementDisplayValue(uab.constants.placement.bottomright) then
                	self:SetStartLocation(uab.constants.placement.bottomright)
				elseif placement == uab.constants.PlacementDisplayValue(uab.constants.placement.left) then
                	self:SetStartLocation(uab.constants.placement.left)
				elseif placement == uab.constants.PlacementDisplayValue(uab.constants.placement.right) then
                	self:SetStartLocation(uab.constants.placement.right)
				end
            end,

            validate = {
				uab.constants.PlacementDisplayValue(uab.constants.placement.topleft),
				uab.constants.PlacementDisplayValue(uab.constants.placement.top),
				uab.constants.PlacementDisplayValue(uab.constants.placement.topright),
				uab.constants.PlacementDisplayValue(uab.constants.placement.bottomleft),
				uab.constants.PlacementDisplayValue(uab.constants.placement.bottom),
				uab.constants.PlacementDisplayValue(uab.constants.placement.bottomright),
				uab.constants.PlacementDisplayValue(uab.constants.placement.left),
				uab.constants.PlacementDisplayValue(uab.constants.placement.right),
			},
        },
	}
	
	return options
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uabSquareLayoutManager.LayoutConfig.prototype:SetStartLocation(value)
    local notify = (value ~= self.settings.startLocation)
        
    self.settings.startLocation = value
    
    if notify then
        self.layout:LayoutSettingsChanged()
    end        
end

function uabSquareLayoutManager.LayoutConfig.prototype:SetGrowClockwise(value)
    local newval
    
    if value and (tonumber(value) == 1 or value == "true") then
        newval = "true"
    else
        newval = "false"
    end
    
    local notify = (newval ~= self.settings.growClockwise)
        
    self.settings.growClockwise = newval

    if notify then
        self.layout:LayoutSettingsChanged()
    end        
end

